package com.muyu.quest.controller;

import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.quest.domain.Node;
import com.muyu.quest.domain.Task;

import javax.annotation.Resource;

import com.muyu.quest.job.TaskJob;
import com.muyu.quest.req.TaskReq;
import com.muyu.quest.resp.TaskResp;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.muyu.quest.service.TaskService;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.web.page.TableDataInfo;

/**
 * 任务Controller
 *
 * @author 2112A
 * @date 2024-08-22
 */
@RestController
@RequestMapping("/task")
public class TaskController extends BaseController
{
    @Resource
    private TaskService taskService;
    @Resource
    private TaskJob taskJob;

    /**
     * 查询任务列表
     */
//    @RequiresPermissions("quest:quest:list")
    @GetMapping("/list")
    public Result<TableDataInfo<TaskResp>> list(TaskReq taskReq)
    {
        startPage();
        List<TaskResp> list = taskService.selectTaskList(taskReq);
        return getDataTable(list);
    }

    /**
     * 获取任务详细信息
     */
//    @RequiresPermissions("quest:quest:query")
    @GetMapping(value = "/selectTaskByTaskCode/{taskCode}")
    public Result<Task> getInfo(@PathVariable("taskCode") String taskCode) {
        return success(taskService.selectTaskByTaskCode(taskCode));
    }

    /**
     * 获取任务详细信息
     */
//    @RequiresPermissions("quest:quest:query")
    @GetMapping(value = "/selectTaskById/{id}")
    public Result<Task> getInfo(@PathVariable("id") Long id) {
        return success(taskService.selectTaskById(id));
    }

    /**
     * 新增任务
     */
//    @RequiresPermissions("quest:quest:quest")
    @PostMapping
    public Result<Integer> add (@Validated @RequestBody Task task) {
        if (taskService.checkTaskCodeUnique(task)) {
            return error("新增参数'" + task.getTaskName() + "'失败，任务已存在");
        }
        String code = UUID.randomUUID().toString().replace("-", "");
        task.setTaskCode(code);
        task.setCreateBy(SecurityUtils.getUsername());
        return toAjax(taskService.save(task));
    }

    /**
     * 修改参数配置
     */
//    @RequiresPermissions("quest:quest:quest")
    @PutMapping
    public Result edit (@Validated @RequestBody Task task) {
        if (!taskService.checkTaskCodeUnique(task)) {
            return error("修改参数'" + task.getTaskName() + "'失败，任务不存在");
        }
        task.setUpdateBy(SecurityUtils.getUsername());
        return toAjax(taskService.updateById(task));
    }

    /**
     * 删除参数配置
     */
//    @RequiresPermissions("system:task:remove")
    @DeleteMapping("/{taskIds}")
    public Result remove (@PathVariable("taskIds") Long[] taskIds) {
        taskService.removeBatch(Arrays.asList(taskIds));
        return success();
    }

    /**
     * 测试任务
     */
    @PostMapping("/testExecute")
    public Result<String> testExecute(@RequestBody List<Node> nodeList) {
        return success(taskService.testExecute(nodeList));
    }

    /**
     * 执行任务
     */
    @PostMapping("/execute/{taskCode}")
    public Result<String> execute(@PathVariable("taskCode") String taskCode) {
        return success(taskService.execute(taskCode));
    }

    /**
     * 手动清理无用任务数据
     */
    @PostMapping("/clearTask")
    public Result clearTask() {
        taskJob.clearTask();
        return success();
    }



}
