package com.muyu.quest.utils;

/**
 * @Author: 胡杨
 * @Name: DispUtils
 * @Description: 节点配置工具
 * @CreatedDate: 2024/9/1 下午4:27
 * @FilePath: com.muyu.quest.utils
 */


import com.muyu.common.core.utils.StringUtils;
import com.muyu.quest.domain.NodeDisposition;
import com.muyu.quest.domain.NodeType;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author: 胡杨
 * @Name: DispUtils
 * @Description: 节点配置工具
 * @CreatedDate: 2024/9/1 下午4:27
 * @FilePath: com.muyu.quest.utils
 */

public class DispUtils {

    /**
     * 筛选节点配置信息
     *
     * @param nodeType     节点类型
     * @param nodeTypeList 节点类型集合
     * @return 配置key数组
     */
    public static String[] getDispKeys(String nodeType, List<NodeType> nodeTypeList) {
        return nodeTypeList
                .stream()
                .filter(type -> StringUtils.equals(type.getNodeTypeCode(), nodeType))
                .map(NodeType::getNodeTypeData)
                .findFirst()
                .orElse("-")
                .split(",");
    }

    /**
     * 整理节点配置信息
     *
     * @param nodeType     节点类型
     * @param nodeTypeList 节点类型集合
     * @param dispList     节点配置集合
     * @return map集合
     */
    public static Map<String, List<NodeDisposition>> getDispMap(
            String nodeType,
            List<NodeType> nodeTypeList,
            List<NodeDisposition> dispList) {
        return getDispMap(getDispKeys(nodeType, nodeTypeList), dispList);
    }

    /**
     * 整理节点配置信息
     *
     * @param dispKeys 配置key
     * @param dispList 配置集合
     * @return map集合
     */
    public static Map<String, List<NodeDisposition>> getDispMap(String[] dispKeys, List<NodeDisposition> dispList) {
        HashMap<String, List<NodeDisposition>> dispMap = new HashMap<>();
        for (String nodeTypeName : dispKeys) {
            dispMap.put(nodeTypeName, dispList.stream()
                    .filter(disp -> StringUtils.equals(disp.getDispKey(), nodeTypeName))
                    .toList());
        }
        return dispMap;
    }

    /**
     * 整理节点配置信息
     *
     * @param dispList 节点配置集合
     * @return map集合
     */
    public static Map<String, List<NodeDisposition>> getDispMap(List<NodeDisposition> dispList) {
        HashMap<String, List<NodeDisposition>> map = new HashMap<>();
        dispList.forEach(disp -> {
            List<NodeDisposition> dispositions = map.get(disp.getDispKey());
            if (dispositions == null || dispositions.isEmpty()) {
                dispositions = new ArrayList<>();
            }
            dispositions.add(disp);
            map.put(disp.getDispKey(), dispositions);
        });
        return map;
    }

}
